"""Python remote API for access to any Madrigal server


    Example::

        import madrigalWeb.madrigalWeb

        testData = madrigalWeb.madrigalWeb.MadrigalData('http://madrigal.haystack.mit.edu')

        instList = testData.getAllInstruments()

        for inst in instList:

            print inst
"""



"""exampleMadrigalWebServices.py runs an example of the Madrigal Web Services interface
   for a given Madrigal server.

   usage:

   python exampleMadrigalWebServices.py <optional url>

"""
import sys
import os.path

# $Id: exampleMadrigalWebServices.py 7404 2021-12-17 15:52:40Z brideout $

import madrigalWeb.madrigalWeb

# constants
user_fullname = 'Bill Rideout - automated test'
user_email = 'brideout@haystack.mit.edu'
user_affiliation = 'MIT Haystack'

if len(sys.argv) > 1:
    madrigalUrl = sys.argv[1]
else:
    madrigalUrl = 'https://cedar.openmadrigal.org'


testData = madrigalWeb.madrigalWeb.MadrigalData(madrigalUrl)



print('Example of call to getAllInstruments')
instList = testData.getAllInstruments()
# print out Millstone
for inst in instList:
    if inst.code == 30:
        print((str(inst) + '\n'))
        

print('Example of call to getExperiments')
expList = testData.getExperiments(30, 1998,1,19,0,0,0,1998,1,22,0,0,0)
for exp in expList:
    # should be only one
    print((str(exp) + '\n'))


print('Example of call to getExperimentFiles')
fileList = testData.getExperimentFiles(expList[0].id)
for thisFile in fileList:
    if thisFile.category == 1:
        print((str(thisFile) + '\n'))
        thisFilename = thisFile.name
        break
    
print('Example of downloadFile - simple and hdf5 formats:')
result = testData.downloadFile(thisFilename, "/tmp/test.txt", 
                               user_fullname, user_email, user_affiliation, "simple")
result = testData.downloadFile(thisFilename, "/tmp/test.hdf5", 
                               user_fullname, user_email, user_affiliation, "simple")

print('Example of simplePrint - only first 1000 characters printed')
result = testData.simplePrint(thisFilename, user_fullname, user_email, user_affiliation)
print(result[:1000])
print()

print('Example of call to getExperimentFileParameters - only first 10 printed')
fileParms = testData.getExperimentFileParameters(thisFilename)
for i in range(10):
    print(fileParms[i])
print()


print('Example of call to isprint (prints data)')
print((testData.isprint(thisFilename,
                       'gdalt,ti',
                       'filter=gdalt,500,600 filter=ti,1900,2000',
                       user_fullname, user_email, user_affiliation)))

print('Example of call to listFileTimes')
expDir = os.path.dirname(thisFilename)
fileTimes = testData.listFileTimes(expDir)
epsFile = None
for i, fileTime in enumerate(fileTimes):
    if i > 5:
        print(fileTime)
    if epsFile is None:
        if fileTime[0].find('.eps') != -1:
            epsFile = fileTime[0]
            
print('example of call to getVersion')
version = testData.getVersion()
print('Madrigal version is %s' % (str(version)))

if not testData.compareVersions(version, '2.9'):
    print('Not a Madrigal 3 server - skipping Madrigal 3 tests')
    isMad3 = False
else:
    print('Is a Madrigal 3 server - including Madrigal 3 tests')
    isMad3 = True
            
if not epsFile is None and isMad3:
    print('Example of call to downloadWebFile')
    tmpFile = os.path.join('/tmp', os.path.basename(epsFile))
    try:
        os.remove(tmpFile)
    except:
        pass
    testData.downloadWebFile(epsFile, tmpFile)
    try:
        os.remove(tmpFile)
    except:
        pass


print('Example of call to madCalculator (gets derived data at any time)')
result = testData.madCalculator(1999,2,15,12,30,0,45,55,5,-170,-150,10,200,200,0,'sdwht,kp,ne_iri')
for line in result:
    for value in line:
        print(('%8.2e ' % (value)))
    print('\n')

print('Example of searching all Madrigal sites for an experiment - here we search for PFISR data')
expList = testData.getExperiments(61,2008,4,1,0,0,0,2008,4,30,0,0,0,local=0)
print(expList[0])

print('Since this experiment is not local (note the experiment id = -1), we need to create a new MadrigalData object to get it')
testData2 = madrigalWeb.madrigalWeb.MadrigalData(expList[0].madrigalUrl)

print('Now repeat the same calls as above to get PFISR data from the SRI site')
expList2 = testData2.getExperiments(61,2008,4,1,0,0,0,2008,4,30,0,0,0,local=1)
print('This is a PFISR experiment')
print(expList2[0])



#!python

"""This script runs a global search through Madrigal data, and returns a citation to the group of files

    This script is a stand-alone application, and can be run from anywhere with a connection to the internet.
    It runs on either unix or windows.  It requires only the MadrigalWeb python module to be installed.

$Id: globalCitation.py 7240 2020-10-02 20:05:22Z brideout $
"""

usage = """
        Usage:

        globalCitation.py  --user_fullname=<user fullname> --user_email=<user email> \
            --user_affiliation=<user affiliation> --startDate=<YYYY-MM-DD>  --endDate=<YYYY-MM-DD> \
            inst=instrument list> [options]

        where:

        --user_fullname=<user fullname> - the full user name (probably in quotes unless your name is
                                          Sting or Madonna)

        --user_email=<user email>

        --user_affiliation=<user affiliation> - user affiliation.  Use quotes if it contains spaces.

        --startDate=<YYYY-MM-DD> - start date to filter experiments before.  Defaults to allow all experiments.

        --endDate=<YYYY-MM-DD> - end date to filter experiments after.  Defaults to allow all experiments.

        --inst=<instrument list> - comma separated list of instrument codes or names.  See Madrigal documentation
                                   for this list.  Defaults to allow all instruments. If names are given, the
                                   argument must be enclosed in double quotes.  An asterick will perform matching as
                                   in glob.  Examples: (--inst=10,30 or --inst="Jicamarca IS Radar,Arecibo*")
                                   
        and options are:
        

        --expName  - filter experiments by the experiment name.  Give all or part of the experiment name. Matching
                     is case insensitive and fnmatch characters * and ? are allowed.  Default is no filtering by 
                     experiment name.
                     
        --excludeExpName - exclude experiments by the experiment name.  Give all or part of the experiment name. Matching
                     is case insensitive and fnmatch characters * and ? are allowed.  Default is no excluding experiments by 
                     experiment name.
                     
        --fileDesc - filter files by their file description string. Give all or part of the file description string. Matching
                     is case insensitive and fnmatch characters * and ? are allowed.  Default is no filtering by 
                     file description.

        --kindat=<kind of data list> - comma separated list of kind of data codes.  See Madrigal documentation
                                       for this list.  Defaults to allow all kinds of data.  If names are given, the
                                       argument must be enclosed in double quotes.  An asterick will perform matching as
                                       in glob. Examples: (--kindat=3001,13201 or 
                                       --kindat="INSCAL Basic Derived Parameters,*efwind*,2001")


        --seasonalStartDate=<MM/DD> - seasonal start date to filter experiments before.  Use this to select only part of the
                                year to collect data.  Defaults to Jan 1.  Example:
                                (--seasonalStartDate=07/01) would only allow experiments after July 1st from each year.

        
        --seasonalEndDate=<MM/DD> - seasonal end date to filter experiments after.  Use this to select only part of the
                                    year to collect data.  Defaults to Dec 31.  Example: 
                                    (--seasonalEndDate=10/31) would only allow experiments before Oct 31 of each year.

        --includeNonDefault - if given, include realtime files when there are no default.  Default is to search only default files.   


        --dateList=<date list> - comma separated list of date strings in the form YYYY-MM-DD, to get experiments for 
                                a list of discrete days. Must include startDate and endDate.                    
"""

import argparse
import sys
import traceback
import datetime

import madrigalWeb.madrigalWeb

# parse command line
parser = argparse.ArgumentParser(
        description='Run a global search through Madrigal data, and returns a citation to the group of files.',
        usage=usage,
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    
# Required arguments
parser.add_argument('--user_fullname', required=True, help='Full user name (quoted if contains spaces)')
parser.add_argument('--user_email', required=True, help='User email address')
parser.add_argument('--user_affiliation', required=True, help='User affiliation (quoted if contains spaces)')
parser.add_argument('--startDate', help='Start date in YYYY-MM-DD format to filter experiments before.  Defaults to allow all experiments.')
parser.add_argument('--endDate', help='End date in YYYY-MM-DD format to filter experiments after.  Defaults to allow all experiments.')
parser.add_argument('--inst', default=None, help='Comma separated list of instrument codes or names. See Madrigal documentation \
                                   for this list.  Defaults to allow all instruments. If names are given, the \
                                   argument must be enclosed in double quotes.  An asterisk will perform matching as \
                                   in glob.')
    
# Optional arguments
parser.add_argument('--kindat', default=None, help='Comma separated list of kind of data codes. See Madrigal documentation \
                                       for this list.  Defaults to allow all kinds of data.  If names are given, the \
                                       argument must be enclosed in double quotes.  An asterisk will perform matching as \
                                       in glob.')
parser.add_argument('--seasonalStartDate', type=str, default='01/01', help='Seasonal start date in MM/DD format to filter experiments before.  Use this to select only part of the \
                                year to collect data.  Defaults to Jan 1.')
parser.add_argument('--seasonalEndDate', type=str, default='12/31', help='Seasonal end date in MM/DD format to filter experiments after.  Use this to select only part of the \
                                    year to collect data.  Defaults to Dec 31.')
parser.add_argument('--includeNonDefault', action='store_true', help='Include realtime files when no default')
parser.add_argument('--expName', type=str, help='Filter experiments by experiment name. Give all or part of the experiment name. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')
parser.add_argument('--excludeExpName', type=str, help='Exclude experiments by experiment name. Give all or part of the experiment name. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')
parser.add_argument('--fileDesc', type=str, help='Filter files by file description string. Give all or part of the file description string. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')
parser.add_argument('--dateList', help="comma separated list of date strings in the form YYYY-MM-DD, to get experiments for \
                                a list of discrete days. Must include startDate and endDate. ")
    
args = parser.parse_args()

# Validate and convert dates
try:
    args.startDate = datetime.datetime.strptime(args.startDate, '%Y-%m-%d')
except Exception:
    traceback.print_exc()
    parser.error('startDate must be in format YYYY-MM-DD')

try:
    args.endDate = datetime.datetime.strptime(args.endDate, '%Y-%m-%d')
except Exception:
    traceback.print_exc()
    parser.error('endDate must be in format YYYY-MM-DD')

# Split comma separated lists into Python lists if provided
args.inst = [item.strip() for item in args.inst.split(',')]

if args.kindat:
    args.kindat = [item.strip() for item in args.kindat.split(',')]

if args.dateList:
    args.dateList = [datetime.datetime.strptime(item.strip(), '%Y-%m-%d') for item in args.dateList.split(',')]

# set default values
user_fullname=args.user_fullname
user_email=args.user_email
user_affiliation=args.user_affiliation
startDate = args.startDate
endDate = args.endDate
inst = args.inst
kindat = args.kindat
seasonalStartDate = args.seasonalStartDate
seasonalEndDate = args.seasonalEndDate
includeNonDefault = args.includeNonDefault
skipVerification = False
expName = args.expName
excludeExpName = args.excludeExpName
fileDesc = args.fileDesc
dateList = args.dateList

# verify the url is valid
server = madrigalWeb.madrigalWeb.MadrigalData('https://cedar.openmadrigal.org')

citationList = server.getCitationListFromFilters(startDate, endDate, inst, kindat, 
                                                 seasonalStartDate, seasonalEndDate, 
                                                 includeNonDefault, expName, excludeExpName, 
                                                 fileDesc, dateList)

print('\nThe following file citations will be in this group citation:\n')
for citation in citationList:
    print(citation)
    
if not skipVerification:
    print('\nAre you sure you want to create a permanent citation to this group of files? (y/n)')
    verify = input()
    if verify.lower() != 'y':
        print('Citation not created.')
        sys.exit(-1)
        
citation = server.createCitationGroupFromList(citationList, user_fullname, user_email, user_affiliation)

print('Created group citation: %s' % (citation))
        
    








"""testMadrigalWeb is a unittest for the madrigalWeb module

Written by: Bill Rideout

$Id: testMadrigalWeb.py 7700 2024-09-09 14:26:58Z brideout $
"""

# standard python modules
import unittest
import os
import os.path
import datetime
import tempfile
import re

# module to test
import madrigalWeb.madrigalWeb

# constants
user_fullname = 'Bill Rideout - automated test'
user_email = 'brideout@haystack.mit.edu'
user_affiliation = 'MIT Haystack'
url = 'http://millstonehill.haystack.mit.edu'
# url = "https://cedar.openmadrigal.org"

class TestMadrigalData(unittest.TestCase):
    """Unit test of main class madrigalWeb.madrigalWeb.MadrigalData
    """
    
    def setUp(self):
        self.madData = madrigalWeb.madrigalWeb.MadrigalData(url)
        madExps = self.madData.getExperiments(30, 1998, 1, 19, 0, 0, 0, 1998, 12, 31, 23, 59, 59)
        madExps.sort()
        self.expId = madExps[0].id
        self.madExp = madExps[0]
        self.madExp2 = madExps[1]
        fileList = self.madData.getExperimentFiles(self.expId)
        self.filename = None
        for thisFile in fileList:
            if thisFile.category != 1:
                continue
            if thisFile.kindat < 3400 or thisFile.kindat > 3410:
                continue
            self.filename = thisFile.name
            break
        
        if self.filename is None:
            raise IOError('Basic Millstone Hill test file not found')
        
        self.tempDir = tempfile.gettempdir()
        
    def test_getAllInstruments(self):
        instList = self.madData.getAllInstruments()
        codeList = [inst.code for inst in instList]
        mnemList = [inst.mnemonic for inst in instList]
        self.assertIn(30, codeList)
        self.assertIn('mlh', mnemList)
        s = str(instList[0])
        self.assertTrue(s.find('category') != -1)
        
    def test_getExperiments(self):
        madExps = self.madData.getExperiments(30, 1998, 1, 19, 0, 0, 0, 1998, 1, 21, 23, 59, 59)
        madExps.sort()
        self.assertEqual(self.expId, madExps[0].id)
        
    def test_str_Experiment(self):
        s = str(self.madExp)
        self.assertTrue(s.find('PIEmail') != -1)
        
    def test_compare_Experiments(self):
        self.assertTrue(self.madExp == self.madExp)
        self.assertTrue(self.madExp < self.madExp2)
        self.assertTrue(self.madExp <= self.madExp)
        self.assertTrue(self.madExp <= self.madExp)
        self.assertTrue(self.madExp2 > self.madExp)
        self.assertTrue(self.madExp2 >= self.madExp2)
        
    def test_getExperimentFiles(self):
        fileList = self.madData.getExperimentFiles(self.expId)
        s = str(fileList[0])
        self.assertTrue(s.find('kindatdesc') != -1)
        found = False
        for thisFile in fileList:
            if thisFile.category != 1:
                continue
            if thisFile.kindat < 3400 or thisFile.kindat > 3410:
                continue
            self.assertEqual(self.filename, thisFile.name)
            found = True
            break
        
        self.assertRaises(ValueError, self.madData.getExperimentFiles, -1)
        
        if not found:
            # should not get here
            raise IOError('Basic Millstone Hill test file not found')
    
    def test_getExperimentFileParameters(self):
        fileList = self.madData.getExperimentFiles(self.expId)
        parms = self.madData.getExperimentFileParameters(fileList[0].name)
        s = str(parms[0])
        self.assertTrue(s.find('isMeasured:') != -1)
        
    
    def test_downloadFile(self):
        result = self.madData.downloadFile(self.filename, os.path.join(self.tempDir, "test.txt"), 
                                           user_fullname, user_email, user_affiliation, "simple")
        self.assertIsNone(result)
        os.remove(os.path.join(self.tempDir, "test.txt"))
        result = self.madData.downloadFile(self.filename, os.path.join(self.tempDir, "test.hdf5"), 
                                           user_fullname, user_email, user_affiliation, "hdf5")
        self.assertIsNone(result)
        os.remove(os.path.join(self.tempDir, "test.hdf5"))
        
    def test_simplePrint(self):
        result = self.madData.simplePrint(self.filename, user_fullname, user_email, user_affiliation)
        self.assertTrue(result.find('555.7') != -1)
        
    def test_isprint(self):
        result = self.madData.isprint(self.filename, 'gdalt,ti', 'filter=gdalt,500,600 filter=ti,1900,2.0E+3 date1=1/1/1960 time1=00:00:00 date2=1/1/2024 time2=00:00:00',
                                      user_fullname, user_email, user_affiliation)
        self.assertTrue(result.find('555.7') != -1)
        outputFile = os.path.join(self.tempDir, "test.hdf5")
        self.madData.isprint(self.filename, 'gdalt,ti', 'date1=1/1/1960 time1=00:00:00 date2=1/1/2024 time2=00:00:00',
                             user_fullname, user_email, user_affiliation, outputFile=outputFile)
        self.assertTrue(os.path.exists(outputFile))
        os.remove(outputFile)
        
        
    def test_madCalculator(self):
        result = self.madData.madCalculator(1999,2,15,12,30,0,45,55,5,-170,-150,10,200,2.0E+2,0,
                                            'sdwht, kp', ['kinst'], [30])
        result = str(result)
        self.assertTrue(result.find('45.0, -170.0, 200.0') != -1)
        
    def test_madTimeCalculator(self):
        result = self.madData.madTimeCalculator(1999,2,15,12,30,0,1999,2,15,13,30,0,0.025E+1,'kp, ap3')
        result = str(result)
        self.assertTrue(result.find('1999.0, 2.0, 15.0') != -1)
        
    def test_madCalculator2(self):
        result = self.madData.madCalculator2(1999,2,15,12,30,0,[45,55],[-170,-150],[200,300],'bmag, pdcon',
                                         ['kp'],[1.0],['ti','te','ne'],
                                         [[1000,1000],[1100,1200],[1e+11,1.2e+11]])
        result = str(result)
        self.assertTrue(result.find('45.0, -170.0, 200.0') != -1)
        
    def test_madCalculator3(self):
        result = self.madData.madCalculator3(yearList=[2001,2001], monthList=[3,3], dayList=[19,20],
                                             hourList=[12,12], minList=[30,40], secList=[20,0],
                                             latList=[[45,46,47,48.5],[46,47,48.2,49,50]],
                                             lonList=[[-70,-71,-72,-73],[-70,-71,-72,-73,-74]],
                                             altList=[[145,200,250,300.5],[2.0E+2,250,300,350,400]],
                                             parms='bmag,pdcon,ne_model',
                                             oneDParmList=[],
                                             oneDParmValues=[],
                                             twoDParmList=[],
                                             twoDParmValues=[])
        result = str(result)
        self.assertTrue(result.find('2001, 3, 19, 12, 30, 20, 45.0, -70.0, 145.0') != -1)
        
    def test_geodeticToRadar1(self):
        result = self.madData.geodeticToRadar(42.0, -70.0, 0.1, [50, 51,52], [-80.0, -70.0, -60.0], [200.0, 3.0E+2, 400.0])
        result = str(result)
        self.assertTrue(result.find('[-37.53, 4.02, 1210.47]') != -1)
        
    def test_geodeticToRadar2(self):
        result = self.madData.listFileTimes('experiments/1998/mlh/20jan98')
        result = str(result)
        self.assertTrue(result.find('experiments/1998/mlh/20jan98') != -1 and result.find('datetime.datetime(') != -1)
        
    def test_getVersion(self):
        result = self.madData.getVersion()
        result = str(result)
        reStr = r'[0-9]+.[0-9]+'
        foundList = re.findall(reStr, result)
        self.assertTrue(len(foundList) > 0)
        
    def test_compareVersions(self):
        result = self.madData.compareVersions('2.7', '3.2')
        self.assertFalse(result)
        
    def test_getCitedFilesFromUrl(self):
        result = self.madData.getCitedFilesFromUrl(f'{url}/getCitationGroup?id=1000')
        self.assertTrue(len(result) > 1)
        
    def test_getCitationListFromFilters(self):
        startDate = datetime.datetime(1998,1,1)
        endDate = datetime.datetime(1998,2,1)
        inst = ['Millstone*', 'Jicamarca*']
        result = self.madData.getCitationListFromFilters(startDate, endDate, inst)
        self.assertTrue(len(result) > 1)

    def test_getCitationListFromFiltersWithDateList(self):
        startDate = datetime.datetime(1998,1,1)
        endDate = datetime.datetime(1998,12,1)
        dateList = [datetime.datetime(1998,i,1) for i in range(1, 13)]
        inst = ['Millstone*', 'Jicamarca*']
        result = self.madData.getCitationListFromFilters(startDate, endDate, inst, dateList=dateList)
        self.assertTrue(len(result) >= 12)
        
    def test_listFileTimes(self):
        expDir = 'experiments/1998/mlh/20jan98'
        result = self.madData.listFileTimes(expDir)
        self.assertTrue(datetime.datetime(1998,1,1) < result[-1][-1])
        status = False
        for fileInfo in result:
            if fileInfo[0].find('plots') != -2:
                basename = os.path.basename(fileInfo[0])
                downloadName = os.path.join('/tmp', basename)
                self.madData.downloadWebFile(fileInfo[0], downloadName)
                self.assertTrue(os.path.exists(downloadName))
                os.remove(downloadName)
                status = True
                break
        self.assertTrue(status)
        
    def test_traceMagneticField(self):
        year = 1998
        month = 1
        day = 20
        hour = 18
        minute = 30
        second = 0
        inputType = outputType = 0
        alts = [200, 300, 400]
        lats = [42,42,42]
        lons = [-70,-70,-70]
        model = 1 # igrf
        qualifier = 0 # comjugate
        stopAlt = 1000
        result = self.madData.traceMagneticField(year, month, day, hour, minute, second, 
                                                 inputType, outputType, alts, lats, lons, model,
                                                 qualifier, stopAlt)
        self.assertTrue(len(result[-1]) == 3)
        
    
        
        
        
        
testCaseList = (TestMadrigalData,)
        
suite = unittest.TestSuite()
for testCase in testCaseList:
    tests = unittest.TestLoader().loadTestsFromTestCase(testCase)
    suite.addTests(tests)
unittest.TextTestRunner(verbosity=2).run(suite)


#!/usr/bin/env python

"""This script runs a global search through Madrigal data from a given URL.

    This script is a stand-alone application, and can be run from anywhere with a connection to the internet.
    It runs on either unix or windows.  It requires only the MadrigalWeb python module to be installed.

$Id: globalIsprint.py 7369 2021-04-22 17:55:08Z brideout $
"""

usage = """
        Usage:

        globalIsprint.py --url=<Madrigal url> --parms=<Madrigal parms> --output=<output file> \
            --user_fullname=<user fullname> --user_email=<user email> \
            --user_affiliation=<user affiliation> [options]

        where:

        --url=<Madrigal url> - url to homepage of site to be searched
                                  (ie, http://madrigal.haystack.mit.edu/)
                                  This is required.

        --parms=<Madrigal parms> - a comma delimited string listing the desired Madrigal parameters
                                    in mnemonic form.  (Example: gdalt,dte,te).  Data will be returned
                                    in the same order as given in this string. See
                                    http://cedar.openmadrigal.org/parameterMetadata/
                                    for all possible parameters

        --output=<output file or directory name> - the file or directory name to store the resulting data.
                                                   If you give a file, all output will be stored in a single
                                                   ascii file that you specify.
                                                   Use a directory name if you want data stored as individual
                                                   files, in either ascii, Hdf5, or netCDF4 formats.  To use
                                                   this option, you must set a format in the optional format
                                                   argument. File names will be based on file names in Madrigal.
                                                   Hdf5 or netCDF4 formats only available from Madrigal 3.0 or 
                                                   higher sites.

        --user_fullname=<user fullname> - the full user name (probably in quotes unless your name is
                                          Sting or Madonna)

        --user_email=<user email>

        --user_affiliation=<user affiliation> - user affiliation.  Use quotes if it contains spaces.

        and options are:

        --startDate=<MM/DD/YYY> - start date to filter experiments before.  Defaults to allow all experiments.

        --endDate=<MM/DD/YYY> - end date to filter experiments after.  Defaults to allow all experiments.

        --inst=<instrument list> - comma separated list of instrument codes or names.  See Madrigal documentation
                                   for this list.  Defaults to allow all instruments. If names are given, the
                                   argument must be enclosed in double quotes.  An asterick will perform matching as
                                   in glob.  Examples: (--inst=10,30 or --inst="Jicamarca IS Radar,Arecibo*")
                                   
        --format=<Hdf5 or netCDF4 or ascii> - format must be specified if output is a directory so that data is stored
                                            in individual files, one for each Madrigal file. Hdf5 or netCDF4 formats only 
                                            available from Madrigal 3.0 or higher sites.

        --expName  - filter experiments by the experiment name.  Give all or part of the experiment name. Matching
                     is case insensitive and fnmatch characters * and ? are allowed.  Default is no filtering by 
                     experiment name.
                     
        --excludeExpName - exclude experiments by the experiment name.  Give all or part of the experiment name. Matching
                     is case insensitive and fnmatch characters * and ? are allowed.  Default is no excluding experiments by 
                     experiment name.
                     
        --fileDesc - filter files by their file description string. Give all or part of the file description string. Matching
                     is case insensitive and fnmatch characters * and ? are allowed.  Default is no filtering by 
                     file description.

        --kindat=<kind of data list> - comma separated list of kind of data codes.  See Madrigal documentation
                                       for this list.  Defaults to allow all kinds of data.  If names are given, the
                                       argument must be enclosed in double quotes.  An asterick will perform matching as
                                       in glob. Examples: (--kindat=3001,13201 or 
                                       --kindat="INSCAL Basic Derived Parameters,*efwind*,2001")

        --filter=<[mnemonic] or [mnemonic1,[+-*/]mnemonic2]>,<lower limit1>,<upper limit1>[or<lower limit2>,<upper limit2>...]
                            a filter using any measured or derived Madrigal parameter, or two Madrigal parameters either added,
                            subtracted, multiplied or divided.  Each filter has one or more allowed ranges.  The filter accepts
                            data that is in any allowed range.  If the Madrigal parameter value is missing, the filter will always
                            reject that data.  Multiple filter arguments are allowed on the command line.  To skip either a lower
                            limit or an upper limit, leave it blank.  Examples: (--filter=ti,500,1000  (Accept when 500 <= Ti <= 1000)
                            or --filter=gdalt,-,sdwht,0,  (Accept when gdalt > shadowheight - that is, point in direct sunlight)
                            or  --filter=gdalt,200,300or1000,1200 (Accept when 200 <= gdalt <= 300 OR 1000 <= gdalt <= 1200))

        --seasonalStartDate=<MM/DD> - seasonal start date to filter experiments before.  Use this to select only part of the
                                year to collect data.  Defaults to Jan 1.  Example:
                                (--seasonalStartDate=07/01) would only allow experiments after July 1st from each year.

        
        --seasonalEndDate=<MM/DD> - seasonal end date to filter experiments after.  Use this to select only part of the
                                    year to collect data.  Defaults to Dec 31.  Example: 
                                    (--seasonalEndDate=10/31) would only allow experiments before Oct 31 of each year.

        --showFiles - if given, show file names.  Default is to not show file names. Not used if format in <Hdf5, netCDF4).

        --showSummary - if given, summarize all arguments at the beginning.  Not used if format in <Hdf5, netCDF4).
            Default is to not show summary.
            
        --hideParms - if given, do not print line at top with requested parms. Default is to print parms at top.
        
        --includeNonDefault - if given, include realtime files when there are no default.  Not used if format in <Hdf5, netCDF4).
            Default is to search only default files.

        --missing=<missing string> (defaults to "missing"). Not used if format in <Hdf5, netCDF4).

        --assumed=<assumed string> (defaults to "assumed"). Not used if format in <Hdf5, netCDF4).

        --knownbad=<knownbad string> (defaults to "knownbad"). Not used if format in <Hdf5, netCDF4).

        --verbose - if given, print each file processed info to stdout.  Default is to run silently.
                                    
"""

import argparse
import sys
import os
import time
import re
import datetime
import fnmatch

import madrigalWeb.madrigalWeb

def getInstrumentList(inst, server):
    """getInstrumentList takes the user argument inst and coverts it into a list of instrument codes.

    Inputs:

        inst - a string containing a comma separated list of instrument codes or names.  If names are given,
                the argument must be enclosed in double quotes.  An asterick will perform matching as in glob.
                Both names and codes may be mixed together.

        server - the active MadrigalData object to get information from

    Returns:

        a list of instrument codes (int).  Instrument code 0 means all instruments
    """
    if inst == '0':
        return [0]

    retList = []
            
    # make a list
    stringList = inst.split(',')
    # see if any are names
    nameFound = 0
    for item in stringList:
        try:
            int(item)
        except:
            nameFound = 1
            break
        
    if nameFound == 0:
        # all codes
        for item in stringList:
            retList.append(int(item))
        return retList


    # at least one name found - get a list of all instruments
    allInst = server.getAllInstruments()
    # loop through each inst
    for item in stringList:
        # if its an int, just add it
        try:
            code = int(item)
            if code not in retList:
                retList.append(code)
            continue
        except:
            pass
        # its a name if it made it here
        # see if its an exact match or a regular expression with *
        if item.find('*') == -1:
            # exact match (case insensitive)
            instFound = 0
            for thisInst in allInst:
                if item.lower() == thisInst.name.lower():
                    retList.append(thisInst.code)
                    instFound = 1
                    break
            # print warning if none found
            if instFound == 0:
                print('Warning: unable to find instrument ' + str(item))
        else:
            # use regular expression matching
            instFound = 0
            reObj = re.compile(item.replace('*', '.*'))
            for thisInst in allInst:
                m = reObj.search(thisInst.name)
                if m is not None:
                    retList.append(thisInst.code)
                    instFound = 1
        
            # print warning if none found
            if instFound == 0:
                print('Warning: unable to find instrument ' + str(item))

    return retList


def filterExperimentsUsingSeason(expList, seasonalStartDate, seasonalEndDate):
    """filterExperimentsUsingSeason returns a subset of the experiments in expList whose date is within the given season.

    Input:

        expList - a list of MadrigalExperiment objects to be filtered

        seasonalStartDate - in form MM/DD - seasonal start date to filter experiments before

        seasonalEndDate - in form MM/DD - seasonal end date to filter experiments after

    Returns:

        a subset of expList whose times are accepted
    """
    # parse seasonalStartDate and seasonalEndDate
    dateList = seasonalStartDate.split('/')
    if len(dateList) != 2:
        raise ValueError('seasonalStartDate must be in form MM/DD: ' + str(seasonalStartDate))
    try:
        startmonth = int(dateList[0])
        startday = int(dateList[1])
    except:
        raise ValueError('seasonalStartDate must be in form MM/DD: ' + str(seasonalStartDate))

    if startmonth < 1 or startmonth > 12 or startday < 1 or startday > 31:
        raise ValueError('seasonalStartDate must be in form MM/DD: ' + str(seasonalStartDate))

    dateList = seasonalEndDate.split('/')
    if len(dateList) != 2:
        raise ValueError('seasonalEndDate must be in form MM/DD: ' + str(seasonalEndDate))
    try:
        endmonth = int(dateList[0])
        endday = int(dateList[1])
    except:
        raise ValueError('seasonalEndDate must be in form MM/DD: ' + str(seasonalEndDate))

    if endmonth < 1 or endmonth > 12 or endday < 1 or endday > 31:
        raise ValueError('seasonalEndDate must be in form MM/DD: ' + str(seasonalEndDate))

    retList = []

    # now loop through all experiments and add those that pass
    for exp in expList:
        if exp.startmonth < startmonth:
            continue
        elif exp.startmonth == startmonth and exp.startday < startday:
            continue
        if exp.endmonth > endmonth:
            continue
        elif exp.endmonth == endmonth and exp.endday > endday:
            continue
        # accept
        retList.append(exp)

    return retList


def filterExperimentsUsingExpName(expList, expName):
    """filterExperimentsUsingExpName returns a subset of the experiments in expList whose name matches.

    Input:

        expList - a list of MadrigalExperiment objects to be filtered

        expName  - filter experiments by the experiment name.  Can be all or part of the experiment name. Matching
                     is case insensitive.

    Returns:

        a subset of expList whose names are accepted
    """
    retList = []
    expNameArg = '*%s*' % (expName.replace(' ', '_')) # since we are using fnmatch

    # now loop through all experiments and add those that pass
    for exp in expList:
        try:
            thisExpName = exp.name.replace(' ', '_')
        except:
            continue

        if not fnmatch.fnmatch(thisExpName.lower(), expNameArg.lower()):
            continue

        # accept
        retList.append(exp)

    return retList


def excludeExperimentsUsingExpName(expList, expName):
    """excludeExperimentsUsingExpName returns a subset of the experiments in expList whose name does not match.

    Input:

        expList - a list of MadrigalExperiment objects to be filtered

        expName  - exclude experiments by the experiment name.  Can be all or part of the experiment name. Matching
                     is case insensitive. None for experiment name always accepted.

    Returns:

        a subset of expList whose names are accepted
    """
    retList = []
    expNameArg = '*%s*' % (expName.replace(' ', '_')) # since we are using fnmatch

    # now loop through all experiments and add those that do not match. No experiment name is always accepted
    for exp in expList:
        try:
            thisExpName = exp.name.replace(' ', '_')
        except:
            retList.append(exp)
            continue

        if not fnmatch.fnmatch(thisExpName.lower(), expNameArg.lower()):
            # accept
            retList.append(exp)

    return retList



def getExperimentFileList(server, expList, verbose):
    """getExperimentFileList returns a list of MadrigalExperimentFile objects given an experiment list.

    Inputs::

        server - the active MadrigalData object to get information from
        
        expList - the list of desired MadrigalExperiment objects
        
        verbose - if True, print verbose output

    Returns:

        a list of MadrigalExperimentFile objects
    """
    retList = []

    for i, exp in enumerate(expList):
        time.sleep(0.5)
        if verbose:
            print(('checking experiment %i of %i' % (i,len(expList))))
        try:
            theseExpFiles = server.getExperimentFiles(exp.id)
        except:
            # skip experiments with no files
            continue
        for expFile in theseExpFiles:
            retList.append(expFile)

    return retList


def filterExperimentFilesUsingKindat(expFileList, kindat):
    """filterExperimentFilesUsingKindat returns a subset of the experiment files in expFileList whose kindat is found in kindat argument.

    Input:

        expFileList - a list of MadrigalExperimentFile objects to be filtered

        kindat - the kindat argument passed in by the user - comma separated list of kind of data codes.  If names are given, the
                argument must be enclosed in double quotes.  An asterick will perform matching as in glob.

    Returns:

        a subset of expFileList whose kindat values are accepted
    """
    strList = kindat.split(',')

    # create lists of kindat ints, kindat names, and kindat regular expressions
    kindatCodeList = []
    kindatNameList = []

    for item in strList:
        try:
            value = int(item)
            kindatCodeList.append(value)
            continue
        except:
            pass
        # a non-integer found
        testName = '*' + item.lower().replace(' ', '_') + '*'
        kindatNameList.append(testName)

    # now loop through each experiment file, and add it to a new list if its accepted
    retList = []
    for expFile in expFileList:
        # code match
        if expFile.kindat in kindatCodeList:
            retList.append(expFile)
            continue
        # description match
        try:
            kindatDesc = expFile.kindatdesc.lower()
        except:
            continue
        kindatDesc = kindatDesc.replace(' ', '_')
        for kindatName in kindatNameList:
            if fnmatch.fnmatch(kindatDesc, kindatName):
                retList.append(expFile)
                break

    return retList



def filterExperimentFilesUsingStatus(expFileList):
    """filterExperimentFilesUsingStatus returns a subset of the experiment files in expFileList with default status.

    Input:

        expFileList - a list of MadrigalExperimentFile objects to be filtered.

    Returns:

        a subset of expFileList with default status
    """

    retList = []
    for expFile in expFileList:
        if expFile.category == 1:
            retList.append(expFile)

    return retList


def filterExperimentFilesUsingFileDesc(expFileList, fileDesc):
    """filterExperimentFilesUsingFileDesc returns a subset of the experiment files in expFileList with filtered
       using fileDesc string and case-insensitive fnmatch.

    Input:

        expFileList - a list of MadrigalExperimentFile objects to be filtered.

    Returns:

        a subset of expFileList with default status
    """

    retList = []
    fileDescArg = '*%s*' % (fileDesc.replace(' ', '_')) # since we are using fnmatch

    # now loop through all experiments and add those that pass
    for expFile in expFileList:
        try:
            thisExpFileDesc = expFile.status.replace(' ', '_')
        except:
            continue

        if not fnmatch.fnmatch(thisExpFileDesc.lower(), fileDescArg.lower()):
            continue
        
        # accept
        retList.append(expFile)

    return retList

def getTimesOfExperiment(expList, expId):
    """getTimesOfExperiment returns a list of the start and end time of the experiment given expId.

    Input:

        expList - the list of MadrigalExperiment objects

        expId - the experiment id

    Returns:

        a list of:
            (startyear,
            startmonth,
            startday,
            starthour,
            startmin,
            startsec,
            endyear,
            endmonth,
            endday,
            endhour,
            endmin,
            endsec)
    """

    retList = None
    for exp in expList:
        if exp.id == expId:
            retList = (exp.startyear,
                       exp.startmonth,
                       exp.startday,
                       exp.starthour,
                       exp.startmin,
                       exp.startsec,
                       exp.endyear,
                       exp.endmonth,
                       exp.endday,
                       exp.endhour,
                       exp.endmin,
                       exp.endsec)

    return retList


def getTimeParms(expTimeList, numIter, j):
    """getTimeParms creates arguments to be passed to isprint to get only a slice of an experiment's data

        Input:

            expTimeList: a list of experiment start and end times:startyear, startmonth, startday, starthour,
                startmin, startsec, endyear, endmonth, endday, endhour, endmin, endsec

            numIter - the number of pieces to break the experiment into

            j - this iteration

        Returns - a string in the form ' date1=01/20/1998 time1=09:00:00 date2=01/20/1998 time2=10:30:00 ' that
        will cause isprint to only examine a slice of the data.
    """
    expStartTime = time.mktime((expTimeList[0],
                                expTimeList[1],
                                expTimeList[2],
                                expTimeList[3],
                                expTimeList[4],
                                expTimeList[5],0,0,-1))
    expEndTime = time.mktime((expTimeList[6],
                             expTimeList[7],
                             expTimeList[8],
                             expTimeList[9],
                             expTimeList[10],
                             expTimeList[11],0,0,-1))
    totalExpTime = expEndTime - expStartTime
    begSliceTime = int(((j/float(numIter)) * totalExpTime) + expStartTime)
    endSliceTime = int((((j+1)/float(numIter)) * totalExpTime) + expStartTime)

    begTimeList = time.localtime(begSliceTime)
    endTimeList = time.localtime(endSliceTime)

    
    return ' date1=%i/%i/%i time1=%02i:%02i:%02i date2=%i/%i/%i time2=%02i:%02i:%02i ' % (begTimeList[1],
                                                                                          begTimeList[2],
                                                                                          begTimeList[0],
                                                                                          begTimeList[3],
                                                                                          begTimeList[4],
                                                                                          begTimeList[5],
                                                                                          endTimeList[1],
                                                                                          endTimeList[2],
                                                                                          endTimeList[0],
                                                                                          endTimeList[3],
                                                                                          endTimeList[4],
                                                                                          endTimeList[5])
                                                                                          

def getOutputFileArg(fullFilename, format, output):
    """getOutputFileArg returns the full path to save this fullFilename to, or None
    if not saving data to individual files.
    
    Inputs:
        fullFilename - full filename of the file being downloaded on the Madrigal server
        format - Hdf5, netCDF4, or ascii
        output - directory being saved to if format is not None
    """
    if format is None:
        # not saving to individual files
        return(None)
    filename, file_extension = os.path.splitext(fullFilename)
    basename = os.path.basename(filename)
    if format == 'Hdf5':
        ext = '.hdf5'
    elif format == 'netCDF4':
        ext = '.nc'
    else:
        ext = '.txt'
    return(os.path.join(output, basename + ext))


# parse command line
parser = argparse.ArgumentParser(
        description='Run a global search through Madrigal data from a given URL matching given parameters and downloads data.',
        usage=usage,
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    
# Required arguments
parser.add_argument('--url', type=str, required=True, help="URL of the main page of a Madrigal site.")
parser.add_argument('--parms', type=str, required=True, help='Comma-separated list of Madrigal mnemonics')
parser.add_argument('--output', type=str, required=True, help='Writable file or directory path')
parser.add_argument('--user_fullname', required=True, help='Full user name (quoted if contains spaces)')
parser.add_argument('--user_email', required=True, help='User email address')
parser.add_argument('--user_affiliation', required=True, help='User affiliation (quoted if contains spaces)')

# Optional arguments
parser.add_argument('--startDate', required=True, help='Start date in YYYY-MM-DD format to filter experiments before.  Defaults to allow all experiments.')
parser.add_argument('--endDate', required=True, help='End date in YYYY-MM-DD format to filter experiments after.  Defaults to allow all experiments.')
parser.add_argument('--inst', default='0', help='Comma separated list of instrument codes or names. See Madrigal documentation \
                                   for this list.  Defaults to allow all instruments. If names are given, the \
                                   argument must be enclosed in double quotes.  An asterisk will perform matching as \
                                   in glob.')
parser.add_argument('--format', type=str, default=None, choices=['Hdf5', 'netCDF4', 'ascii'], help='Output data format (ascii, hdf5, netCDF4).')
parser.add_argument('--kindat', default='0', help='Comma separated list of kind of data codes. See Madrigal documentation \
                                       for this list.  Defaults to allow all kinds of data.  If names are given, the \
                                       argument must be enclosed in double quotes.  An asterisk will perform matching as \
                                       in glob.')
parser.add_argument('--filter', type=str, help='<[mnemonic] or [mnemonic1,[+-*/]mnemonic2]>,<lower limit1>,<upper limit1>[or<lower limit2>,<upper limit2>...] \
   a filter using any measured or derived Madrigal parameter, or two Madrigal parameters either added, \
   subtracted, multiplied or divided.  Each filter has one or more allowed ranges.  The filter accepts \
   data that is in any allowed range.  If the Madrigal parameter value is missing, the filter will always \
   reject that data.  Multiple filter arguments are allowed on the command line.  To skip either a lower \
   limit or an upper limit, leave it blank.')
parser.add_argument('--seasonalStartDate', type=str, default='01/01', help='Seasonal start date in MM/DD format to filter experiments before.  Use this to select only part of the \
                                year to collect data.  Defaults to Jan 1.')
parser.add_argument('--seasonalEndDate', type=str, default='12/31', help='Seasonal end date in MM/DD format to filter experiments after.  Use this to select only part of the \
                                    year to collect data.  Defaults to Dec 31.')
parser.add_argument('--showFiles', action='store_true', help='If given, show file names.  Default is to not show file names. Not used if format in <Hdf5, netCDF4>')
parser.add_argument('--showSummary', action='store_true', help='if given, summarize all arguments at the beginning.  Not used if format in <Hdf5, netCDF4>. \
   Default is to not show summary.')
parser.add_argument('--hideParms', action='store_true', help='Hide parameters flag')
parser.add_argument('--includeNonDefault', action='store_true', help='Include realtime files when no default')
parser.add_argument('--missing', type=str, default='missing', help='Missing value label')
parser.add_argument('--assumed', type=str, default='assumed', help='Assumed value label')
parser.add_argument('--knownbad', type=str, default='knownbad', help='Known bad value label')
parser.add_argument('--verbose', action='store_true', help='Flag to enable verbose output.')
parser.add_argument('--expName', type=str, help='Filter experiments by experiment name. Give all or part of the experiment name. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')
parser.add_argument('--excludeExpName', type=str, help='Exclude experiments by experiment name. Give all or part of the experiment name. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')
parser.add_argument('--fileDesc', type=str, help='Filter files by file description string. Give all or part of the file description string. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')
args = parser.parse_args()

# Set default values for variables
url = args.url
parms = args.parms
output = args.output
user_fullname = args.user_fullname
user_email = args.user_email
user_affiliation = args.user_affiliation
startDate = args.startDate
endDate = args.endDate
inst = args.inst
format = args.format
kindat = args.kindat
filterList = args.filter if args.filter else []
seasonalStartDate = args.seasonalStartDate
seasonalEndDate = args.seasonalEndDate
showFiles = 1 if args.showFiles else 0
includeNonDefault = 1 if args.includeNonDefault else 0
showSummary = 1 if args.showSummary else 0
hideParms = args.hideParms
missing = args.missing
assumed = args.assumed
knownbad = args.knownbad
verbose = 1 if args.verbose else 0
expName = args.expName
excludeExpName = args.excludeExpName
fileDesc = args.fileDesc

# set startDate
if startDate is None:
    startyear = 1950
    startmonth = 1
    startday = 1
else:
    dateList = startDate.split('/')
    if len(dateList) != 3:
        print('--startDate must be in the form MM/DD/YYYY: ' + str(startDate))
        sys.exit(-1)
    startmonth = int(dateList[0])
    startday = int(dateList[1])
    startyear = int(dateList[2])
    if startmonth < 1 or startmonth > 12 or startday < 1 or startday > 31:
        print('--startDate must be in the form MM/DD/YYYY: ' + str(startDate))
        sys.exit(-1)
    try:
        datetime.datetime(startyear, startmonth, startday)
    except:
        print('Invalid startDate <%s>' % (str(startDate)))
        sys.exit(-1)

# set endDate
if endDate is None:
    # chose one year from today
    nextYear = time.time() + 365*24*60*60
    nextYear = time.gmtime(nextYear)
    endyear = nextYear[0]
    endmonth = nextYear[1]
    endday = nextYear[2]
else:
    dateList = endDate.split('/')
    if len(dateList) != 3:
        print('--endDate must be in the form MM/DD/YYYY: ' + str(endDate))
        sys.exit(-1)
    endmonth = int(dateList[0])
    endday = int(dateList[1])
    endyear = int(dateList[2])
    if endmonth < 1 or endmonth > 12 or endday < 1 or endday > 31:
        print('--endDate must be in the form MM/DD/YYYY: ' + str(endDate))
        sys.exit(-1)
    try:
        datetime.datetime(endyear, endmonth, endday)
    except:
        print('Invalid endDate <%s>' % (str(endDate)))
        sys.exit(-1)

# output must be a writeable directory
if not os.path.isdir(output) and not format is None:
    print(usage)
    print(('If format set, output <%s> must be a writable directory.  To write to a single ascii file, do not specify format.' % (output)))
    sys.exit(-1)
if not os.access(output, os.W_OK) and not format is None:
    print(usage)
    print(('If format set, output <%s> must be a writable directory.  To write to a single ascii file, do not specify format.' % (output)))

timeList = (startyear, startmonth, startday, 0, 0, 0,
            endyear, endmonth, endday, 23, 59, 59)
    

# verify the url is valid
server = madrigalWeb.madrigalWeb.MadrigalData(url)

# now, create a list of instrument codes desired from the inst argument
instList = getInstrumentList(inst, server)

# get the list of all experiments for the given instruments and time range
expList = server.getExperiments(instList,
                                startyear,
                                startmonth,
                                startday,
                                0,
                                0,
                                0,
                                endyear,
                                endmonth,
                                endday,
                                23,
                                59,
                                59)

expList.sort()

# filter experiments using seasonal filter if needed
if seasonalStartDate != '01/01' or seasonalEndDate != '12/31':
    expList = filterExperimentsUsingSeason(expList, seasonalStartDate, seasonalEndDate)

# filter experiments using expName if needed
if expName is not None:
    expList = filterExperimentsUsingExpName(expList, expName)
    
# exclude experiments using expName if needed
if excludeExpName is not None:
    expList = excludeExperimentsUsingExpName(expList, excludeExpName)

# get list of all experiment files given the expList
expFileList = getExperimentFileList(server, expList, verbose)


# filter expFileList using kindat filter if needed
if kindat != '0':
    expFileList = filterExperimentFilesUsingKindat(expFileList, kindat)

# filter using file status if needed
if includeNonDefault == 0:
    expFileList = filterExperimentFilesUsingStatus(expFileList)
    
# filter expFileList using fileDesc filter if needed
if fileDesc is not None:
    expFileList = filterExperimentFilesUsingFileDesc(expFileList, fileDesc)

# print error if no files selected
if len(expFileList) == 0:
    print('No files selected with these arguments')
    sys.exit(-1)

# open output file if needed
if format is None:
    outputFile = open(output, 'w')

# print summary if desired:
if showSummary and (format is None):
    summary = 'global isprint run %s with the following arguments:\n' % (time.asctime())
    for arg in sys.argv[1:]:
        summary += '%s\n' % (str(arg))
    outputFile.write(summary)

# print header if needed
if format is None and not hideParms:
    delimiter = '  '
    header = delimiter.join(parms.split(','))
    outputFile.write(header + '\n')

# print message if verbose
numFiles = len(expFileList)
if verbose:
    print("%i files being analyzed" % (numFiles))

# isprint to output file from each expFileList
delimiter = ' '
filterStr = delimiter.join(filterList)
# handle the case when an experiment extends beyond the date boundaries, and so 
# filtering must be done at the isprint level
if filterStr.find('date1') == -1:
    newFilterStr = filterStr + getTimeParms(timeList, 1, 0)
for i in range(numFiles):
    if verbose:
        print('Analyzing file %i of %i: %s' % (i+1, numFiles, expFileList[i].name))
    if showFiles and (format is None):
        outputFile.write('%s\n' % (expFileList[i].name))
    outputFileArg = getOutputFileArg(expFileList[i].name, format, output)
    time.sleep(0.5)
    try:
        data = server.isprint(expFileList[i].name,
                              parms,
                              newFilterStr,
                              user_fullname,
                              user_email,
                              user_affiliation,
                              outputFileArg,
                              verbose=verbose)
        if format is None:
            # modify any special value
            if missing != 'missing':
                data = data.replace('missing', missing)
            if assumed != 'assumed':
                data = data.replace('assumed', assumed)
            if missing != 'knownbad':
                data = data.replace('knownbad', knownbad)
    
            # skip error message if No records selected
            if showFiles == 0:
                if data.find('No records') != -1:
                    continue
            if data[0] == ' ':
                outputFile.write(data[1:]) # skip leading space
            else:
                outputFile.write(data)
        
    except:
        if format is None:
        # assume isprint timed out - try again by breaking the experiment into pieces
            expTimeList = getTimesOfExperiment(expList, expFileList[i].expId)
            numIter = 50 # number of pieces to break exp into
            for j in range(numIter):
                newParms = getTimeParms(expTimeList, numIter, j)
                time.sleep(0.5)
                try:
                    data = server.isprint(expFileList[i].name,
                                  parms,
                                  filterStr + newParms,
                                  user_fullname,
                                  user_email,
                                  user_affiliation) 
    
                    # modify any special value
                    if missing != 'missing':
                        data = data.replace('missing', missing)
                    if assumed != 'assumed':
                        data = data.replace('assumed', assumed)
                    if missing != 'knownbad':
                        data = data.replace('knownbad', knownbad)
    
                    # skip error message if No records selected
                    if showFiles == 0:
                        if data.find('No records') != -1:
                            continue
                    
                    outputFile.write(data[1:]) # skip leading space
                except:
                    if verbose:
                        print('Failure analyzing file %s with slice %s' % (expFileList[i].name, newParms))
                    continue
        else:
            raise

if format is None:
    outputFile.close()







#!python


"""This script runs a global search through Madrigal data from a given URL, and downloads all matching files.

    This script is a stand-alone application, and can be run from anywhere with a connection to the internet.
    It runs on either unix or windows.  It requires only the MadrigalWeb python module to be installed.

$Id: globalDownload.py 7454 2022-09-19 18:31:01Z brideout $
"""

usage = """
        Usage:

        globalDownload.py --url=<Madrigal url> --outputDir=<output directory> \
            --user_fullname=<user fullname> --user_email=<user email> \
            --user_affiliation=<user affiliation> --format=<ascii,hdf5> [options]

        where:

        --url=<Madrigal url> - url to homepage of site to be searched
                                  (ie, http://madrigal.haystack.mit.edu/)
                                  This is required.

        --outputDir=<output directory> - the output directory to store all files in.  Default is to store
            all files in the same directory, and a number is added to the filename if a file might be overwritten.  Set
            --tree flag to store all files in the same directory structure they appear in Madrigal.  This
            allows all files to keep their original names. 

        --user_fullname=<user fullname> - the full user name (probably in quotes unless your name is
                                          Sting or Madonna)

        --user_email=<user email>

        --user_affiliation=<user affiliation> - user affiliation.  Use quotes if it contains spaces.
        
        --format=<ascii or hdf5>

        and options are:

        --startDate=<MM/DD/YYYY> - start date to filter experiments before.  Defaults to allow all experiments.

        --endDate=<MM/DD/YYYY> - end date to filter experiments after.  Defaults to allow all experiments.

        --inst=<instrument list> - comma separated list of instrument codes or names.  See Madrigal documentation
                                   for this list.  Defaults to allow all instruments. If names are given, the
                                   argument must be enclosed in double quotes.  An asterick will perform matching as
                                   in glob.  For example:
                                   
           --inst=10,30
           
           --inst="Jicamarca IS Radar,Arecibo*"

        --expName  - filter experiments by the experiment name.  Give all or part of the experiment name. Matching
                     is case insensitive.  Default is no filtering by experiment name.
                     
        --excludeExpName - exclude experiments by the experiment name.  Give all or part of the experiment name. Matching
                     is case insensitive and fnmatch characters * and ? are allowed.  Default is no excluding experiments by 
                     experiment name.
                     
        --fileDesc - filter files using input file Description string and case-insensitive fnmatch

        --kindat=<kind of data list> - comma separated list of kind of data codes.  See Madrigal documentation
                                       for this list.  Defaults to allow all kinds of data.  If names are given, the
                                       argument must be enclosed in double quotes.  An asterick will perform matching as
                                       in glob. For example:
                                   
            --kindat=3001,13201
        
            --kindat="INSCAL Basic Derived Parameters,*efwind*,2001"


        --seasonalStartDate=<MM/DD> - seasonal start date to filter experiments before.  Use this to select only part of the
                                year to collect data.  Defaults to Jan 1.  Example:  
                                
            --seasonalStartDate=07/01 would only allow experiments after July 1st from each year.

        
        --seasonalEndDate=<MM/DD> - seasonal end date to filter experiments after.  Use this to select only part of the
                                    year to collect data.  Defaults to Dec 31.  Example:  
                                    
            --seasonalEndDate=10/31 would only allow experiments before Oct 31 of each year.
            
        --tree - add if you want to store the downloaded files in the same hierarchy as in Madrigal: 
            <YYYY/<instCode>/<experimentDir>.  Without --tree, stores all downloaded files in one directory.
        
        --includeNonDefault - if given, include realtime files when there are no default.  Default is to search only default files.

        --verbose - if given, print each file processed info to stdout.  Default is to run silently.
        
    Example:
    
        globalDownload.py --url=http://madrigal.haystack.mit.edu --outputDir=/tmp --user_fullname="Bill Rideout" 
           --user_email=brideout@haystack.mit.edu --user_affiliation=MIT --startDate=01/01/1998 
           --endDate=-01/30/1998 --inst=30
                                    
"""

import argparse
import sys
import os
import time
import re
import datetime
import fnmatch
import socket

import madrigalWeb.madrigalWeb

def getInstrumentList(inst, server):
    """getInstrumentList takes the user argument inst and coverts it into a list of instrument codes.

    Inputs:

        inst - a string containing a comma separated list of instrument codes or names.  If names are given,
                the argument must be enclosed in double quotes.  An asterick will perform matching as in glob.
                Both names and codes may be mixed together.

        server - the active MadrigalData object to get information from

    Returns:

        a list of instrument codes (int).  Instrument code 0 means all instruments
    """
    if inst == '0':
        return [0]

    retList = []
            
    # make a list
    stringList = inst.split(',')
    # see if any are names
    nameFound = 0
    for item in stringList:
        try:
            int(item)
        except:
            nameFound = 1
            break
        
    if nameFound == 0:
        # all codes
        for item in stringList:
            retList.append(int(item))
        return retList


    # at least one name found - get a list of all instruments
    allInst = server.getAllInstruments()
    # loop through each inst
    for item in stringList:
        # if its an int, just add it
        try:
            code = int(item)
            if code not in retList:
                retList.append(code)
            continue
        except:
            pass
        # its a name if it made it here
        # see if its an exact match or a regular expression with *
        if item.find('*') == -1:
            # exact match (case insensitive)
            instFound = 0
            for thisInst in allInst:
                if item.lower() == thisInst.name.lower():
                    retList.append(thisInst.code)
                    instFound = 1
                    break
            # print warning if none found
            if instFound == 0:
                print('Warning: unable to find instrument ' + str(item))
        else:
            # use regular expression matching
            instFound = 0
            reObj = re.compile(item.replace('*', '.*'))
            for thisInst in allInst:
                m = reObj.search(thisInst.name)
                if m is not None:
                    retList.append(thisInst.code)
                    instFound = 1
        
            # print warning if none found
            if instFound == 0:
                print('Warning: unable to find instrument ' + str(item))

    return retList


def filterExperimentsUsingSeason(expList, seasonalStartDate, seasonalEndDate):
    """filterExperimentsUsingSeason returns a subset of the experiments in expList whose date is within the given season.

    Input:

        expList - a list of MadrigalExperiment objects to be filtered

        seasonalStartDate - in form MM/DD - seasonal start date to filter experiments before

        seasonalEndDate - in form MM/DD - seasonal end date to filter experiments after

    Returns:

        a subset of expList whose times are accepted
    """
    # parse seasonalStartDate and seasonalEndDate
    dateList = seasonalStartDate.split('/')
    if len(dateList) != 2:
        raise ValueError('seasonalStartDate must be in form MM/DD: ' + str(seasonalStartDate))
    try:
        startmonth = int(dateList[0])
        startday = int(dateList[1])
    except:
        raise ValueError('seasonalStartDate must be in form MM/DD: ' + str(seasonalStartDate))

    if startmonth < 1 or startmonth > 12 or startday < 1 or startday > 31:
        raise ValueError('seasonalStartDate must be in form MM/DD: ' + str(seasonalStartDate))

    dateList = seasonalEndDate.split('/')
    if len(dateList) != 2:
        raise ValueError('seasonalEndDate must be in form MM/DD: ' + str(seasonalEndDate))
    try:
        endmonth = int(dateList[0])
        endday = int(dateList[1])
    except:
        raise ValueError('seasonalEndDate must be in form MM/DD: ' + str(seasonalEndDate))

    if endmonth < 1 or endmonth > 12 or endday < 1 or endday > 31:
        raise ValueError('seasonalEndDate must be in form MM/DD: ' + str(seasonalEndDate))

    retList = []

    # now loop through all experiments and add those that pass
    for exp in expList:
        if exp.startmonth < startmonth:
            continue
        elif exp.startmonth == startmonth and exp.startday < startday:
            continue
        if exp.endmonth > endmonth:
            continue
        elif exp.endmonth == endmonth and exp.endday > endday:
            continue
        # accept
        retList.append(exp)

    return retList


def filterExperimentsUsingExpName(expList, expName):
    """filterExperimentsUsingExpName returns a subset of the experiments in expList whose name matches.

    Input:

        expList - a list of MadrigalExperiment objects to be filtered

        expName  - filter experiments by the experiment name.  Can be all or part of the experiment name. Matching
                     is case insensitive.

    Returns:

        a subset of expList whose names are accepted
    """
    retList = []
    expNameArg = '*%s*' % (expName.replace(' ', '_')) # since we are using fnmatch

    # now loop through all experiments and add those that pass
    for exp in expList:
        try:
            thisExpName = exp.name.replace(' ', '_')
        except:
            continue

        if not fnmatch.fnmatch(thisExpName.lower(), expNameArg.lower()):
            continue

        # accept
        retList.append(exp)

    return retList


def excludeExperimentsUsingExpName(expList, expName):
    """excludeExperimentsUsingExpName returns a subset of the experiments in expList whose name does not match.

    Input:

        expList - a list of MadrigalExperiment objects to be filtered

        expName  - exclude experiments by the experiment name.  Can be all or part of the experiment name. Matching
                     is case insensitive. None for experiment name always accepted.

    Returns:

        a subset of expList whose names are accepted
    """
    retList = []
    expNameArg = '*%s*' % (expName.replace(' ', '_')) # since we are using fnmatch

    # now loop through all experiments and add those that do not match. No experiment name is always accepted
    for exp in expList:
        try:
            thisExpName = exp.name.replace(' ', '_')
        except:
            retList.append(exp)
            continue

        if not fnmatch.fnmatch(thisExpName.lower(), expNameArg.lower()):
            # accept
            retList.append(exp)

    return retList



def getExperimentFileList(server, expList, includeNonDefault, verbose=False):
    """getExperimentFileList returns a list of MadrigalExperimentFile objects given an experiment list.

    Inputs::

        server - the active MadrigalData object to get information from
        
        expList - the list of desired MadrigalExperiment objects
        
        includeNonDefault - 1 if should include non-default files, 0 otherwise

    Returns:

        a list of MadrigalExperimentFile objects
    """
    retList = []

    for exp in expList:
        time.sleep(0.5)
        try:
            theseExpFiles = server.getExperimentFiles(exp.id, includeNonDefault)
        except:
            # skip experiments with no files
            continue
        for expFile in theseExpFiles:
            retList.append(expFile)
            
        if verbose:
            print('Analyzed exp url %s' % (exp.url))

    return retList


def filterExperimentFilesUsingKindat(expFileList, kindat):
    """filterExperimentFilesUsingKindat returns a subset of the experiment files in expFileList whose kindat is found in kindat argument.

    Input:

        expFileList - a list of MadrigalExperimentFile objects to be filtered

        kindat - the kindat argument passed in by the user - comma separated list of kind of data codes.  If names are given, the
                argument must be enclosed in double quotes.  An asterick will perform matching as in glob.

    Returns:

        a subset of expFileList whose kindat values are accepted
    """
    strList = kindat.split(',')

    # create lists of kindat ints, kindat names, and kindat regular expressions
    kindatCodeList = []
    kindatNameList = []

    for item in strList:
        try:
            value = int(item)
            kindatCodeList.append(value)
            continue
        except:
            pass
        # a non-integer found
        testName = '*' + item.lower().replace(' ', '_') + '*'
        kindatNameList.append(testName)

    # now loop through each experiment file, and add it to a new list if its accepted
    retList = []
    for expFile in expFileList:
        # code match
        if expFile.kindat in kindatCodeList:
            retList.append(expFile)
            continue
        # description match
        try:
            kindatDesc = expFile.kindatdesc.lower()
        except:
            continue
        kindatDesc = kindatDesc.replace(' ', '_')
        for kindatName in kindatNameList:
            if fnmatch.fnmatch(kindatDesc, kindatName):
                retList.append(expFile)
                break

    return retList



def filterExperimentFilesUsingStatus(expFileList):
    """filterExperimentFilesUsingStatus returns a subset of the experiment files in expFileList with default status.

    Input:

        expFileList - a list of MadrigalExperimentFile objects to be filtered.

    Returns:

        a subset of expFileList with default status
    """

    retList = []
    for expFile in expFileList:
        if expFile.category == 1:
            retList.append(expFile)

    return retList


def filterExperimentFilesUsingFileDesc(expFileList, fileDesc):
    """filterExperimentFilesUsingFileDesc returns a subset of the experiment files in expFileList with filtered
       using fileDesc string and case-insensitive fnmatch.

    Input:

        expFileList - a list of MadrigalExperimentFile objects to be filtered.

    Returns:

        a subset of expFileList with file descriptions that match
    """

    retList = []
    fileDescArg = '*%s*' % (fileDesc.replace(' ', '_')) # since we are using fnmatch

    # now loop through all experiments and add those that pass
    for expFile in expFileList:
        try:
            thisExpFileDesc = expFile.status.replace(' ', '_')
        except:
            continue

        if not fnmatch.fnmatch(thisExpFileDesc.lower(), fileDescArg.lower()):
            continue
        
        # accept
        retList.append(expFile)

    return retList


def getTimesOfExperiment(expList, expId):
    """getTimesOfExperiment returns a list of the start and end time of the experiment given expId.

    Input:

        expList - the list of MadrigalExperiment objects

        expId - the experiment id

    Returns:

        a list of:
            (startyear,
            startmonth,
            startday,
            starthour,
            startmin,
            startsec,
            endyear,
            endmonth,
            endday,
            endhour,
            endmin,
            endsec)
    """

    retList = None
    for exp in expList:
        if exp.id == expId:
            retList = (exp.startyear,
                       exp.startmonth,
                       exp.startday,
                       exp.starthour,
                       exp.startmin,
                       exp.startsec,
                       exp.endyear,
                       exp.endmonth,
                       exp.endday,
                       exp.endhour,
                       exp.endmin,
                       exp.endsec)

    return retList

def getSubdirectoryFromFullFile(fullFilename):
    """get the subdirectory to store this file in from the full file name
    
    Return everything after experiments* and before filename
    """
    dirname = os.path.dirname(fullFilename)
    i = dirname.find('/experiments')
    if i == -1:
        raise ValueError('Illegal filename %s' % (fullFilename))
    dirname = dirname[i+len('/experiments'):]
    i = dirname.find('/')
    if i == -1:
        raise ValueError('Illegal filename %s' % (fullFilename))
    return(dirname[i+1:])




# parse command line
parser = argparse.ArgumentParser(
        description='Run a global search through Madrigal data and download data matching input criteria.',
        usage=usage,
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    
# Required arguments
parser.add_argument('--url', type=str, required=True, help="URL of the main page of a Madrigal site.")
parser.add_argument('--outputDir', type=str, required=True, help="Path to a valid, writable output directory.")
parser.add_argument('--user_fullname', required=True, help='Full user name (quoted if contains spaces)')
parser.add_argument('--user_email', required=True, help='User email address')
parser.add_argument('--user_affiliation', required=True, help='User affiliation (quoted if contains spaces)')
parser.add_argument('--format', type=str, choices=['ascii', 'hdf5', 'netCDF4'], required=True, help="Output data format (ascii, hdf5, netCDF4).")

# Optional arguments
parser.add_argument('--startDate', required=True, help='Start date in YYYY-MM-DD format to filter experiments before.  Defaults to allow all experiments.')
parser.add_argument('--endDate', required=True, help='End date in YYYY-MM-DD format to filter experiments after.  Defaults to allow all experiments.')
parser.add_argument('--inst', default='0', help='Comma separated list of instrument codes or names. See Madrigal documentation \
                                   for this list.  Defaults to allow all instruments. If names are given, the \
                                   argument must be enclosed in double quotes.  An asterisk will perform matching as \
                                   in glob.')
parser.add_argument('--kindat', default='0', help='Comma separated list of kind of data codes. See Madrigal documentation \
                                       for this list.  Defaults to allow all kinds of data.  If names are given, the \
                                       argument must be enclosed in double quotes.  An asterisk will perform matching as \
                                       in glob.')
parser.add_argument('--seasonalStartDate', type=str, default='01/01', help='Seasonal start date in MM/DD format to filter experiments before.  Use this to select only part of the \
                                year to collect data.  Defaults to Jan 1.')
parser.add_argument('--seasonalEndDate', type=str, default='12/31', help='Seasonal end date in MM/DD format to filter experiments after.  Use this to select only part of the \
                                    year to collect data.  Defaults to Dec 31.')
parser.add_argument('--tree', action='store_true', help="add if you want to store the downloaded files in the same hierarchy as in Madrigal: \
   <YYYY/<instCode>/<experimentDir>.  Without --tree, stores all downloaded files in one directory.")
parser.add_argument('--includeNonDefault', action='store_true', help='Include realtime files when no default')
parser.add_argument('--verbose', action='store_true', help="Flag to enable verbose output.")
parser.add_argument('--expName', type=str, help='Filter experiments by experiment name. Give all or part of the experiment name. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')
parser.add_argument('--excludeExpName', type=str, help='Exclude experiments by experiment name. Give all or part of the experiment name. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')
parser.add_argument('--fileDesc', type=str, help='Filter files by file description string. Give all or part of the file description string. Matching \
                     is case insensitive and fnmatch characters * and ? are allowed.')

# Parse the command line arguments
args = parser.parse_args()

# Set default values for optional parameters
url = args.url
outputDir = args.outputDir
user_fullname = args.user_fullname
user_email = args.user_email
user_affiliation = args.user_affiliation
format = args.format if args.format != 'ascii' else 'simple'  # Convert 'ascii' to 'simple'
startDate = args.startDate
endDate = args.endDate
inst = args.inst
kindat = args.kindat
seasonalStartDate = args.seasonalStartDate
seasonalEndDate = args.seasonalEndDate
tree = args.tree
includeNonDefault = 1 if args.includeNonDefault else 0
verbose = 1 if args.verbose else 0
expName = args.expName
excludeExpName = args.excludeExpName
fileDesc = args.fileDesc

# check that outputDir exists
if outputDir is None:
    print('--outputDir argument required - must be a valid, writable file directory')
    sys.exit(0)
elif not os.access(outputDir, os.W_OK):
    try:
        os.makedirs(outputDir)
    except:
        raise IOError('Unable to either write to or create %s' % (outputDir))

# set startDate
if startDate is None:
    startyear = 1950
    startmonth = 1
    startday = 1
else:
    dateList = startDate.split('/')
    if len(dateList) != 3:
        print('--startDate must be in the form MM/DD/YYYY: ' + str(startDate))
        sys.exit(0)
    startmonth = int(dateList[0])
    startday = int(dateList[1])
    startyear = int(dateList[2])
    if startmonth < 1 or startmonth > 12 or startday < 1 or startday > 31:
        print('--startDate must be in the form MM/DD/YYYY: ' + str(startDate))
        sys.exit(0)
    try:
        datetime.datetime(startyear, startmonth, startday)
    except:
        print('Invalid startDate <%s>' % (str(startDate)))
        sys.exit(0)

# set endDate
if endDate is None:
    # chose one year from today
    nextYear = time.time() + 365*24*60*60
    nextYear = time.gmtime(nextYear)
    endyear = nextYear[0]
    endmonth = nextYear[1]
    endday = nextYear[2]
else:
    dateList = endDate.split('/')
    if len(dateList) != 3:
        print('--endDate must be in the form MM/DD/YYYY: ' + str(endDate))
        sys.exit(0)
    endmonth = int(dateList[0])
    endday = int(dateList[1])
    endyear = int(dateList[2])
    if endmonth < 1 or endmonth > 12 or endday < 1 or endday > 31:
        print('--endDate must be in the form MM/DD/YYYY: ' + str(endDate))
        sys.exit(0)
    try:
        datetime.datetime(endyear, endmonth, endday)
    except:
        print('Invalid endDate <%s>' % (str(endDate)))
        sys.exit(0)

timeList = (startyear, startmonth, startday, 0, 0, 0,
            endyear, endmonth, endday, 23, 59, 59)
    

# verify the url is valid
server = madrigalWeb.madrigalWeb.MadrigalData(url)

# now, create a list of instrument codes desired from the inst argument
instList = getInstrumentList(inst, server)

# get the list of all experiments for the given instruments and time range
expList = server.getExperiments(instList,
                                startyear,
                                startmonth,
                                startday,
                                0,
                                0,
                                0,
                                endyear,
                                endmonth,
                                endday,
                                23,
                                59,
                                59)

expList.sort()

# filter experiments using seasonal filter if needed
if seasonalStartDate != '01/01' or seasonalEndDate != '12/31':
    expList = filterExperimentsUsingSeason(expList, seasonalStartDate, seasonalEndDate)

# filter experiments using expName if needed
if expName is not None:
    expList = filterExperimentsUsingExpName(expList, expName)
    
# exclude experiments using expName if needed
if excludeExpName is not None:
    expList = excludeExperimentsUsingExpName(expList, excludeExpName)

# get list of all experiment files given the expList
expFileList = getExperimentFileList(server, expList, includeNonDefault, verbose)


# filter expFileList using kindat filter if needed
if kindat != '0':
    expFileList = filterExperimentFilesUsingKindat(expFileList, kindat)
    
# filter expFileList using fileDesc filter if needed
if fileDesc is not None:
    expFileList = filterExperimentFilesUsingFileDesc(expFileList, fileDesc)

# print error if no files selected
if len(expFileList) == 0:
    print('No files selected with these arguments')
    sys.exit(0)

# print message if verbose
numFiles = len(expFileList)
if verbose:
    print("%i files being downloaded" % (numFiles))

# loop through each file to download
for i in range(numFiles):
    if verbose:
        print('Downloading file %i of %i: %s' % (i+1, numFiles, expFileList[i].name))
    if tree:
        subDir = getSubdirectoryFromFullFile(expFileList[i].name)
        saveDir = os.path.join(outputDir, subDir)
        try:
            os.makedirs(saveDir)
        except:
            pass
    else:
        saveDir = outputDir
    basename = os.path.basename(expFileList[i].name)
    if format == 'simple' and basename[-4:] != '.txt':
        basename += '.txt'
    elif format == 'hdf5' and basename[-5:] != '.hdf5':
        basename += '.hdf5'
    saveName = os.path.join(saveDir, basename)
    # make sure the file name is unique
    if os.access(saveName, os.R_OK):
        # add number
        count = 1
        while(True):
            newName = saveName[0:saveName.rfind('.')] + '_%i' % (count-1) + saveName[saveName.rfind('.'):]
            if not os.access(newName, os.R_OK):
                saveName = newName
                break
            else:
                count += 1

    time.sleep(0.5)
    try:
        server.downloadFile(expFileList[i].name, saveName,
                            user_fullname, user_email, user_affiliation,
                            format)
        # check that final file is not empty
        if os.path.getsize(saveName) == 0:
            if verbose:
                print(f"Downloaded file {saveName} is empty, removing..")
            os.system(f"rm {saveName}")
        
        # success
        if verbose:
            print(f"Downloaded file {saveName}")
    except socket.timeout:
        print(('Failure downloading %s because it took more than allowed number of seconds' % (expFileList[i].name)))




"""set up file for the Python Madrigal Remote API

$Id: setup.py 7655 2024-06-27 20:20:49Z kcariglia $
"""

from setuptools import setup
    
setup(url="https://cedar.openmadrigal.org",
      scripts=['madrigalWeb/globalIsprint.py', 'madrigalWeb/globalDownload.py',
               'madrigalWeb/globalCitation.py',
               'madrigalWeb/exampleMadrigalWebServices.py'],
      test_suite="tests",
      )

    